/*
 * Copyright 2010-2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams;

/**
 * This alternative to JUnit-rules can be used under CallbackParams but regular
 * JUnit-rules can of course also be used during normal circumstances.
 * This alternative is useful when JUnit-rules do not work for some reason, such
 * as when the test-class is a JUnit-3.x style test-class or if a
 * {@link org.callbackparams.junit4.WrappedRunner @WrappedRunner}
 * annotation specifies a runner that does not support JUnit-rules.
 *
 * @see <a href="http://callbackparams.org/guide.html#wrappedrunner">Guide on @WrappedRunner</a>
 * @author Henrik Kaipe
 */
public abstract class AdaptiveRule {
    public interface TestRun {
        Object executeTestMethod() throws Throwable;
    }

    public abstract Object evaluate(TestRun testRun) throws Throwable;
}
