/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.bdd.support;

import org.callbackparams.support.ClassConstantModifyingVisitor;
import org.callbackparams.support.TemplateBasedClassBuilder;

/**
 * @author Henrik Kaipe
 */
public class SuperClassChanger {

    private final Class<?> classTemplate;
    private final ClassConstantModifyingVisitor glueAsSuperClass;

    public SuperClassChanger(Class<?> classTemplate) {
        this.classTemplate = classTemplate;
        this.glueAsSuperClass = new ClassConstantModifyingVisitor(
                classTemplate.getSuperclass(), DummyGlueClass.class);
    }

    public <C> Class<? extends C> asSubclassOf(Class<C> newSuperClass) {
        Class<? extends C> glueClass = buildGlueClass(newSuperClass);

        return TemplateBasedClassBuilder
                .newBuilder(glueClass.getClassLoader(), classTemplate)
                .applyVisitor(glueAsSuperClass)
                .setNewClassName(newSuperClass.getName() + "$BDD$")
                .wrapClass().getWrappedClass().asSubclass(glueClass);
    }

    private <C> Class<? extends C> buildGlueClass(Class<C> superClass) {
        ClassConstantModifyingVisitor
                superClassChangingVisitor = new ClassConstantModifyingVisitor(
                DummyGlueClass.class.getSuperclass(), superClass);

        return TemplateBasedClassBuilder
                .newBuilder(superClass.getClassLoader(), DummyGlueClass.class)
                .applyVisitor(superClassChangingVisitor)
                .wrapClass().getWrappedClass().asSubclass(superClass);
    }
}
