/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.combine;

import java.util.ArrayList;
import java.util.List;

/**
 * A collection of arrays of callback-values. CombineStrategy
 * implementations will create their callback-records in a manner that makes
 * each callback-record contain exactly one callback-value from each array.
 * 
 * @author Henrik Kaipe
 */
public class ValuesCollection {
    
    List valuesArrays = new ArrayList();

    /**
     * Attaches the specified array of callback-values at the end of this
     * ValuesCollection.
     */
    public void add(Object[] valuesArray) {
        if (0 < valuesArray.length) {
            valuesArrays.add(valuesArray);
        }
    }
    
    /**
     * Returns the array that contains the callback-values available for
     * the specified index of the callback-records created from this
     * ValuesCollection instance.
     */
    public Object[] get(int index) {
        return (Object[]) valuesArrays.get(index);
    }
    
    /**
     * Returns the number of arrays of callback-values in this
     * ValuesCollection instance.
     */
    public int size() {
        return valuesArrays.size();
    }
}
