/*
 * Copyright 2012-2013 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.internal.template;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import org.callbackparams.support.ExceptionUtil;

/**
 * @author Henrik Kaipe
 */
public abstract class AnnotationSpecifiedCallbacks {

    private final static Constructor methodParamConstructor =
            locateMethodParamConstructor();
    private final static Constructor fieldConstructor =
            locateFieldConstructor();

    private static Constructor locateMethodParamConstructor() {
        String actualAnnotationSpecifiedCallbacksClassName =
                AnnotationSpecifiedCallbacks.class.getName()
                .replaceFirst("A", "annotation.A");
        try {
            return Class.forName(actualAnnotationSpecifiedCallbacksClassName)
                    .getConstructor(new Class[] {Method.class, int.class});
        } catch (UnsupportedClassVersionError detectionOf_JDK1_4) {
            return null;
        } catch (Exception shouldNeverHappen) {
            throw ExceptionUtil.unchecked(shouldNeverHappen);
        }
    }

    private static Constructor locateFieldConstructor() {
        if (null == methodParamConstructor) {
            return null;
        } else {
            try {
                return methodParamConstructor.getDeclaringClass()
                        .getConstructor(new Class[]{Field.class});
            } catch (NoSuchMethodException shouldNeverHappen) {
                throw ExceptionUtil.unchecked(shouldNeverHappen);
            }
        }
    }

    static AnnotationSpecifiedCallbacks forCallbackRef(Method m, int paramIndex) {
        return newInstance(methodParamConstructor,
                new Object[] {m, new Integer(paramIndex)});
    }

    static AnnotationSpecifiedCallbacks forCallbackRef(Field f) {
        return newInstance(fieldConstructor, new Object[] {f});
    }

    private static AnnotationSpecifiedCallbacks newInstance(
            Constructor constructor, Object[] constructorArgs) {
        if (null == constructor) {
            return jdk14Impl();
        } else {
            try {
                return (AnnotationSpecifiedCallbacks)
                        constructor.newInstance(constructorArgs);
            } catch (Exception shouldNeverHappen) {
                throw ExceptionUtil.unchecked(shouldNeverHappen);
            }
        }
    }

    private static AnnotationSpecifiedCallbacks jdk14Impl() {
        /*
         * This functionality is unsupported for JDK-1.4, so the
         * returned instance is simply a dummy instance.
         */
        return new AnnotationSpecifiedCallbacks() {
            protected Object asCallback(Object callbackValue) {
                return null;
            }
        };
    }

    abstract protected Object asCallback(Object callbackValue);
}
