/*
 * Copyright 2010-2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.internal.template;

import org.callbackparams.support.DefaultValue;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.callbackparams.CallbackControlPanel;

/**
 * Wraps an array of all combined values that are to be used for a
 * single test-run. This array is also referred to as a callback-record
 * throughout the CallbackParams documentation - hence the name of this class.
 *
 * @author Henrik Kaipe
 */
public class CallbackRecord {

    private final List callbackRecord;
    private final Map latestCallbackResults = new IdentityHashMap();
    private final CallbackControlPanel ccp = new CallbackControlPanel() {
        public List getCurrentCallbackRecord() {
            return callbackRecord;
        }
        public Map getLatestCallbackResults() {
            return latestCallbackResults;
        }
    };

    public CallbackRecord(List callbackRecord) {
        this.callbackRecord = callbackRecord;
    }

    public CallbackControlPanel getCallbackControlPanel() {
        return ccp;
    }

    public InvocationHandler newCallbackInvoker(final CallbackRef callbackRef) {
        return new InvocationHandler() {
            
            public Object invoke(Object o, Method m, Object[] args)
            throws Throwable {
                if (false == m.isAccessible()) {
                    m.setAccessible(true);
                }
                try {
                    latestCallbackResults.clear();

                    for (Iterator i = callbackRecord.iterator(); i.hasNext();) {
                        final Object callbackRecordElement = i.next();
                        if (null == callbackRecordElement) {
                            continue;
                        }
                        
                        final Object targetCallbackInstance =
                                callbackRef.asCallback(callbackRecordElement);
                        if (null != targetCallbackInstance) {
                            latestCallbackResults.put(
                                    callbackRecordElement,
                                    m.invoke(targetCallbackInstance, args));
                        }
                    }
                    return DefaultValue.forType(m.getReturnType());
                    
                } catch (final InvocationTargetException ite) {
                    throw ite.getTargetException();
                }
            }
        };
    }
}
