/*
 * Copyright 2010-2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.internal.template;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Iterator;
import org.callbackparams.internal.CallbackContext;
import org.callbackparams.internal.CallbackContextFactory;
import org.callbackparams.internal.CallbackTestClassReloader;

/**
 * The functionality used here was initially implemented to make integration
 * with the class-reloading mock-framework "Powermock" as seamless as
 * possible but this will hopefully also comply well with other
 * class-reloading test-frameworks.
 *
 * @author Henrik Kaipe
 */
class ThirdPartyReloadingWorkaround {

    private static final String callbackParamsClassReloaderClassName =
            CallbackTestClassReloader.class.getName();

    static ThirdPartyReloadingWorkaround getInstance() {
        return new ThirdPartyReloadingWorkaround();
    }

    /**
     * The key to finding the callback-setup for this test-run is to find the
     * {@link CallbackTestClassReloader} class-loader in which the callback-
     * methods and callback-values were specified and then translate these
     * to their equivalents in the class-loader of the argument
     * "reloadedTestClass".<br/><br/>
     * The algorithm for finding the class-loader is
     * currently tailored in order to work in case the test-class was reloaded
     * by Powermock. This will likely work for a few other situations as well
     * but here might be a need for future improvements in case there comes a
     * situation that is much different from the Powermock case.
     */
    void resurrectSetup(Class reloadedTestClass)
    throws Exception {
        CallbackContext testRunContext = CallbackContextFactory
                .forClassLoader(reloadedTestClass.getClassLoader());

        ClassLoader callbackDefLoader = reloadedTestClass.getClassLoader();
        do {
            callbackDefLoader = callbackDefLoader.getClass().getClassLoader();
        } while (false == callbackDefLoader.getClass().getName()
                .equals(callbackParamsClassReloaderClassName));
        CallbackContext callbackDefContext = CallbackContextFactory
                .forClassLoader(callbackDefLoader);
        for (Iterator i = callbackDefContext.iterateMethods() ; i.hasNext() ;) {
            final Method m = (Method) i.next();
            testRunContext.addMethodParams(m);
        }
        for (Iterator i = callbackDefContext.iterateFields() ; i.hasNext() ;) {
            final Field f = (Field) i.next();
            testRunContext.addFieldToInject(f);
        }
        testRunContext.setCallbackRecord(
                callbackDefContext.getCallbackRecord());
    }
}
