/*
 * Copyright 2012-2013 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.internal.template.annotation;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import org.callbackparams.ext.WrappingCallbackImpls;
import org.callbackparams.wrap.legacy.Wrapping;

/**
 * @author Henrik Kaipe
 */
public class AnnotationSpecifiedCallbacks<T>
extends org.callbackparams.internal.template.AnnotationSpecifiedCallbacks {

    private final Class<T> callbackType;

    private AnnotationSpecifiedCallbacks(Class<?> callbackType) {
        this.callbackType = (Class<T>) callbackType;
    }

    public AnnotationSpecifiedCallbacks(Method m, int paramIndex) {
        this(m.getParameterTypes()[paramIndex]);
    }

    public AnnotationSpecifiedCallbacks(Field f) {
        this(f.getType());
    }

    @Override
    protected T asCallback(Object callbackValue) {
        WrappingCreator<T> wrappingCreator = new WrappingCreator<T>(
                callbackType, callbackValue, classSpecifiedImpls(callbackValue));
        for (WrappingClassLocator wrappingLocator : WrappingClassLocator.values()) {
            T wrappingCallback = wrappingCreator.createCallbackInstance(
                    wrappingLocator.locateClasses(callbackValue),
                    wrappingLocator.canBeSubclassedByClassSpecifiedImpl());
            if (null != wrappingCallback) {
                return wrappingCallback;
            }
        }
        return null;
    }

    private Class<? extends Wrapping>[] classSpecifiedImpls(Object value) {
        WrappingCallbackImpls classSpecifiedImpls =
                value.getClass().getAnnotation(WrappingCallbackImpls.class);
        return null == classSpecifiedImpls
                ? new Class[0] : classSpecifiedImpls.value();
    }
}
