/*
 * Copyright 2012 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.internal.template.annotation;

import org.callbackparams.CallbackFactory;
import org.callbackparams.support.ExceptionUtil;
import org.callbackparams.wrap.legacy.Wrapping;
import org.callbackparams.wrap.legacy.WrappingLegacySupport;

/**
 * @author Henrik Kaipe
 */
class ValidatingWrappingFactory<T> {

    private static final String wrappedValueMethodName =
            Wrapping.class.getDeclaredMethods()[0].getName();

    private final Object wrappedValue;

    ValidatingWrappingFactory(Object wrappedValue) {
        this.wrappedValue = wrappedValue;
    }

    boolean isCompatibleWrappedValue(Class<? extends Wrapping> wrappingClass) {
        try {
            return wrappingClass
                    .getMethod(wrappedValueMethodName)
                    .getReturnType()
                    .isAssignableFrom(wrappedValue.getClass());
        } catch (NoSuchMethodException shouldNeverHappen) {
            throw ExceptionUtil.unchecked(shouldNeverHappen);
        }
    }

    @SuppressWarnings("unchecked")
    T newInstance(Class<? extends Wrapping> wrappingClass) {
        Object wrapperInstance;
        try {
            wrapperInstance = WrappingLegacySupport
                    .wrapSingle(wrappingClass, wrappedValue);
        } catch (Throwable error) {
            throw ExceptionUtil.unchecked(error);
        }
        try {
            return (T) (wrapperInstance instanceof CallbackFactory
                    ? ((CallbackFactory)wrapperInstance).getCallback()
                    : wrapperInstance);
        } catch (Exception callbackFactoryExceptionsAreTreatedAsNullCallbacks) {
            return null;
        }
    }
}
