/*
 * Copyright 2010-2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.junit4;

import java.lang.annotation.Annotation;

/**
 * A marker super-class that is used to recognize
 * {@link org.junit.rules.MethodRule} implementations that are provided by
 * this framework. These rule-classes do not implement
 * MethodRule directly but have their actual implementation implement it in
 * case the rule is used under JUnit-4.7 or later.
 * The idea is to not cause too many pattern variations because of tests that
 * are originally created for different versions of JUnit by having these
 * rule-implementations also provide a rather hardwired API for accomplishing
 * the rule features manually. It can also be useful when
 * wrapping a third-party runner implementation that does not reproduce
 * the functionality of {@link org.junit.runners.BlockJUnit4ClassRunner}.
 *
 * @author Henrik Kaipe
 */
class DependentOnRulesSupport {

    static final Class<? extends Annotation> RULE_ANNOTATION =
            possiblySupportedRuleAnnotation();

    boolean JUNIT_RULES_ARE_SUPPORTED = null != RULE_ANNOTATION;

    private static Class<? extends Annotation> possiblySupportedRuleAnnotation()
    {
        try {
            return Class.forName("org.junit.Rule").asSubclass(Annotation.class);
        } catch (ClassNotFoundException JUnit_4_7_or_later_UNAVAILABLE) {
            return null;
        }
    }
}
