/*
 * Copyright 2010-2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.junit4;

import org.callbackparams.junit4.version.CallbackRunListener;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import junit.framework.Test;
import junit.framework.TestCase;
import org.callbackparams.internal.CallbackTestClassReloader;
import org.callbackparams.junit3.CallbackTestCase;
import org.callbackparams.junit3.support.RebyteInfo;
import org.callbackparams.junit4.version.Version;
import org.junit.runner.Runner;
import org.junit.runner.notification.RunNotifier;

import static org.callbackparams.junit4.RebyteInfo.isParameterizedCallback;

/**
 * @author Henrik Kaipe
 */
class Junit3RunWrapper extends RunWrapper {

    /* Initial value might be updated by isCallbackTestCase(...) */
    private boolean isCallbackTestCase = false;

    /* Initial value might be updated by isSuiteMethodTest(...) */
    private boolean isSuiteMethodTest = false;

    @Override
    void run(CallbackRunListener callbackRunListener) {
        if (isCallbackTestCase) {
            {
                /*
                 * This block is a somewhat ugly workaround to get
                 * callback-context into a proper state */
                CallbackTestClassReloader classReloader =
                        (CallbackTestClassReloader)
                        reloadedTestClass.getClassLoader();
                classReloader.getCallbackContext();
            }

            RunNotifier outerNotifier = new RunNotifier();
            outerNotifier.addListener(callbackRunListener);
            createRunner().run(outerNotifier);
        } else {
            super.run(callbackRunListener);
        }
    }

    Runner createDefaultRunner() {
        if (isSuiteMethodTest) {
            return Version.newJunit38SuiteRunner(reloadedTestClass);
        } else {
            return Version.newJunit38Runner(reloadedTestClass);
        }
    }

    /**
     * @return true if the test-class seems to be based on the JUnit-3.x API and
     * therewith makes this run-wrapper initialize properly; false otherwise, ie
     * this run-wrapper can not used and the test-class must be run with a
     * JUnit-4.x run-wrapper.
     */
    boolean tryInitTestClass(Class<?> testClass) throws ClassNotFoundException {
        if (isSuiteMethodTest(testClass) || isCallbackTestCase(testClass)
                || TestCase.class.isAssignableFrom(testClass)) {
            CallbackTestClassReloader classReloader =
                    CallbackTestClassReloader.newInstance(
                    testClass, WrappedRunnerReloadFilter.getInstance(),
                    new RebyteInfo() {
                public void putNoargProxyMethod(
                        String proxyMethodName, Method callbackProxiedMethod) {
                    noargProxyMethodMap.put(
                            proxyMethodName, callbackProxiedMethod.getName());
                }
                @Override
                public boolean isCallbackInjectField(Field f) {
                    return isParameterizedCallback(f);
                }
            });
            this.reloadedTestClass =
                    classReloader.loadClass(testClass.getName());
            return true;
        } else {
            return false;
        }
    }

    private boolean isSuiteMethodTest(Class<?> testClass) {
        try {
            Method suiteMethod = testClass.getMethod("suite");
            this.isSuiteMethodTest =
                    Test.class.isAssignableFrom(suiteMethod.getReturnType());
            return this.isSuiteMethodTest;
        } catch (NoSuchMethodException ex) {
            return false;
        }
    }

    private boolean isCallbackTestCase(Class<?> testClass) {
        isCallbackTestCase = CallbackTestCase.class.isAssignableFrom(testClass);
        return isCallbackTestCase;
    }
}
