/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.junit4.version;

import org.junit.runner.Description;
import org.junit.runner.Result;
import org.junit.runner.notification.Failure;
import org.junit.runner.notification.RunNotifier;

/**
 * @author Henrik Kaipe
 */
class CallbackRunListener_4_4_OrEarlier extends CallbackRunListener {

    final RunNotifier actualNotifier;

    CallbackRunListener_4_4_OrEarlier(RunNotifier actualNotifier) {
        this.actualNotifier = actualNotifier;
    }

    @Override
    public void testFailure(Failure failure) throws Exception {
        actualNotifier.fireTestFailure(modify(failure));
    }
    @Override
    public void testFinished(Description description) throws Exception {
        actualNotifier.fireTestFinished(modify(description));
    }
    @Override
    public void testIgnored(Description description) throws Exception {
        actualNotifier.fireTestIgnored(modify(description));
    }
    @Override
    public void testRunFinished(Result result) throws Exception {
        throw new IllegalStateException("Method test-run finished is not " +
                "supposed to be invoked from within the runner: " + result);
    }
    @Override
    public void testRunStarted(Description description) throws Exception {
        actualNotifier.fireTestRunStarted(modify(description));
    }
    @Override
    public void testStarted(Description description) throws Exception {
        actualNotifier.fireTestStarted(modify(description));
    }


}
