/*
 * Copyright 2011-2012 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.wrap.legacy;

import org.callbackparams.support.AnnotationInjector;

/**
 * @author Henrik Kaipe
 */
class WrappingTemplate extends DummySuperClass implements Wrapping {

    static final ThreadLocal mostRecentlyInjectedWrappedValue =
            new ThreadLocal();

    /**
     * Can have three different states ...
     *   null - means that wrapped value has not yet been set
     *   this - means that wrapped value is null
     *   any other value is a non-null wrapped value
     */
    private Object wrappedValue;

    protected WrappingTemplate() {
        /*
         * Will make sure this instance is properly initiated
         * - in case it has not already been taken care of by a super-class:
         */
        wrappedValueDefaultImpl();
    }

    public Object wrappedValue() {
        return wrappedValueDefaultImpl();
    }

    private Object wrappedValueDefaultImpl() {
        if (null == wrappedValue) {
            /* Injected value is not yet securly wrapped ... */
            wrappedValue = mostRecentlyInjectedWrappedValue.get();
            if (null == wrappedValue) {
                wrappedValue = this;
            } else {
                AnnotationInjector.injectAnnotations(this, wrappedValue);
            }
        }
        return this == wrappedValue /* represents 'null' as wrappedValue() */
                ? null
                : wrappedValue;
    }
}
